package com.supervision.rasa.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.model.RasaModelInfo;
import com.supervision.rasa.config.ThreadPoolExecutorConfig;
import com.supervision.rasa.pojo.vo.RasaCmdArgumentVo;
import com.supervision.rasa.service.RasaCmdService;
import com.supervision.rasa.util.PortUtil;
import com.supervision.service.RasaModeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;
import java.util.function.Predicate;

@Service
@Slf4j
@RequiredArgsConstructor
public class RasaCmdServiceImpl implements RasaCmdService {


    @Value("${rasa.models-path}")
    private String modelsPath;

    @Value("${rasa.endpoints}")
    private String endpoints;

    @Value("${rasa.config}")
    private String config;

    @Value("${rasa.data-path}")
    private String dataPath;


    @Value("${rasa.shell-env:/bin/bash}")
    private String shellEnv;

    @Value("${rasa.train-shell}")
    private String trainShell;

    @Value("${rasa.run-shell}")
    private String runShell;

    @Value("${rasa.shell.work:/home/rasa_manage/}")
    private String shellWork;

    private final RasaModeService rasaModeService;

    private final  String TRAN_SUCCESS_MESSAGE = "Your Rasa model is trained and saved at";
    private final  String RUN_SUCCESS_MESSAGE = "Rasa server is up and running";

    @Override
    @Transactional
    public String trainExec(RasaCmdArgumentVo argument) throws ExecutionException, InterruptedException, TimeoutException {

        String domain = dataPath+"domain.yml";
        List<String> cmds = ListUtil.toList(shellEnv, trainShell,config,dataPath,domain,modelsPath);
        cmds.add(argument.getFixedModelName());

        log.info("trainExec cmd : {}",StrUtil.join(" ",cmds));

        List<String> outMessage = execCmd(cmds, s -> false, 300);

        //保存 模型信息
        RasaModelInfo rasaModelInfo = new RasaModelInfo();
        rasaModelInfo.setModelId(argument.getModelId());
        rasaModelInfo.setTranStatus(trainIsSuccess(outMessage)?1:0);
        rasaModelInfo.setServerStatus(-1);
        rasaModeService.saveOrUpdateByModelId(rasaModelInfo);

        return String.join("\r\n",outMessage);
    }



    @Override
    public String runExec(RasaCmdArgumentVo argument) throws ExecutionException, InterruptedException, TimeoutException {


        // 1. 查找可用端口
        int port = PortUtil.findUnusedPort(5050, 100000,rasaModeService.listActivePort());
        log.info("runExec findUnusedPort is : {}",port);

        // 2. 运行模型
        String mPath = modelsPath+argument.getFixedModelName()+".tar.gz";
        List<String> cmds = ListUtil.toList(shellEnv, runShell,mPath,endpoints,String.valueOf(port));

        log.info("runExec cmd : {}",StrUtil.join(" ",cmds));

        List<String> outMessageList = execCmd(cmds, s -> StrUtil.isNotBlank(s) && s.contains(RUN_SUCCESS_MESSAGE), 300);

        // 3. 更新模型信息
        RasaModelInfo rasaModelInfo = new RasaModelInfo();
        rasaModelInfo.setModelId(argument.getModelId());
        rasaModelInfo.setPort(port);
        rasaModelInfo.setServerStatus(runIsSuccess(outMessageList)?1:0);
        rasaModeService.saveOrUpdateByModelId(rasaModelInfo);

        return String.join("\r\n",outMessageList);
    }

    @Override
    public void test() {
        RasaModelInfo rasaModelInfo = new RasaModelInfo();
        rasaModelInfo.setId("1");
        rasaModelInfo.setModelId("1");
        rasaModelInfo.setTranStatus(1);
        rasaModeService.saveOrUpdate(rasaModelInfo);
    }


    private List<String> execCmd(List<String> cmds, Predicate<String> endPredicate, long timeOut) throws InterruptedException, ExecutionException, TimeoutException {
        ProcessBuilder processBuilder = new ProcessBuilder(cmds);
        processBuilder.directory(new File(shellWork));
        processBuilder.redirectErrorStream(true);

        ThreadPoolExecutor instance = ThreadPoolExecutorConfig.getInstance();
        Future<List<String>> future = instance.submit(() -> {
            Process process = processBuilder.start();
            InputStream inputStream = process.getInputStream();
            Reader reader = new InputStreamReader(inputStream, "UTF-8");
            BufferedReader bufferedReader = new BufferedReader(reader);
            List<String> outString = new ArrayList<>();
            String resultLines = bufferedReader.readLine();
            while( resultLines != null) {
                resultLines = bufferedReader.readLine();
                log.info("resultLines:{}",resultLines);
                outString.add(resultLines);
                if (endPredicate.test(resultLines)){
                    break;
                }
            }
            bufferedReader.close();
            return new ArrayList<>();
        });

        return future.get(timeOut, TimeUnit.SECONDS);
    }




    private  boolean trainIsSuccess(List<String> messageList){

        return containKey(messageList,TRAN_SUCCESS_MESSAGE);
    }


    private  boolean runIsSuccess(List<String> messageList){

        return containKey(messageList,RUN_SUCCESS_MESSAGE);
    }

    private boolean containKey(List<String> messageList,String keyWord){

        if (CollectionUtil.isEmpty(messageList)){
            return false;
        }
        if (StrUtil.isEmpty(keyWord)){
            return false;
        }
        return messageList.stream().anyMatch(s->StrUtil.isNotEmpty(s) && s.contains(keyWord));
    }

}
